@@ -2,11 +2,10 @@
  *  File:             generate.c
  *  Project Main:     boxes.c
  *  Date created:     June 23, 1999 (Wednesday, 20:10h)
- *  Author:           Copyright (C) 1999 Thomas Jensen
- *                    tsjensen@stud.informatik.uni-erlangen.de
- *  Version:          $Id: generate.c,v 1.8 1999/08/22 11:36:45 tsjensen Exp tsjensen $
+ *  Author:           Copyright (C) 1999 Thomas Jensen <boxes@thomasjensen.com>
+ *  Version:          $Id: generate.c,v 1.11 2006/07/22 19:21:13 tsjensen Exp $
  *  Language:         ANSI C
- *  World Wide Web:   http://home.pages.de/~jensen/boxes/
+ *  World Wide Web:   http://boxes.thomasjensen.com/
  *  Purpose:          Box generation, i.e. the drawing of boxes
  *
  *  Remarks: o This program is free software; you can redistribute it and/or
@@ -25,6 +24,17 @@
  *  Revision History:
  *
  *    $Log: generate.c,v $
+ *    Revision 1.11  2006/07/22 19:21:13  tsjensen
+ *    Applied patch by Christoph Dreyer to support unexpansion of leading tabs
+ *    Added ability to retain existing tabs
+ *
+ *    Revision 1.10  2006/07/12 05:42:28  tsjensen
+ *    Updated email and web addresses in comment header
+ *
+ *    Revision 1.9  1999-08-31 08:37:01-07  tsjensen
+ *    Applied Joe Zbiciak's patches to remove all snprintf()s and variants
+ *    Replaced snprintf() calls with calls to concat_strings() in the process
+ *
  *    Revision 1.8  1999/08/22 11:36:45  tsjensen
  *    Removed const from 2nd parameter of justify_line() function
  *
@@ -68,8 +78,10 @@
 #include "tools.h"
 #include "generate.h"
 
+
 static const char rcsid_generate_c[] =
-    "$Id: generate.c,v 1.8 1999/08/22 11:36:45 tsjensen Exp tsjensen $";
+    "$Id: generate.c,v 1.11 2006/07/22 19:21:13 tsjensen Exp $";
+
 
 
 
@@ -856,6 +868,7 @@ int output_box (const sentry_t *thebox)
     size_t nol = thebox[BRIG].height;    /* number of output lines */
     char   trailspc[LINE_MAX+1];
     char  *indentspc;
+    int    indentspclen;
     size_t vfill, vfill1, vfill2;        /* empty lines/columns in box */
     size_t hfill;
     char  *hfill1, *hfill2;              /* space before/after text */
@@ -867,6 +880,8 @@ int output_box (const sentry_t *thebox)
     size_t skip_start;                   /* lines to skip for box top */
     size_t skip_end;                     /* lines to skip for box bottom */
     size_t skip_left;                    /* true if left box part is to be skipped */
+    int    ntabs, nspcs;                 /* needed for unexpand of tabs */
+    char  *restored_indent;
 
     #ifdef DEBUG
         fprintf (stderr, "Padding used: left %d, top %d, right %d, bottom %d\n",
@@ -877,14 +892,31 @@ int output_box (const sentry_t *thebox)
     /*
      *  Create string of spaces for indentation
      */
+    indentspc = NULL;
+    ntabs = nspcs = indentspclen = 0;
     if (opt.design->indentmode == 'b') {
-        indentspc = (char *) malloc (input.indent+1);
+        if (opt.tabexp == 'u') {
+            ntabs = input.indent / opt.tabstop;
+            nspcs = input.indent % opt.tabstop;
+            indentspclen = ntabs + nspcs;
+        }
+        else {
+            indentspclen = input.indent;
+        }
+        indentspc = (char *) malloc (indentspclen + 1);
         if (indentspc == NULL) {
             perror (PROJECT);
             return 1;
         }
-        memset (indentspc, (int)' ', input.indent);
-        indentspc[input.indent] = '\0';
+
+        if (opt.tabexp == 'u') {
+            memset (indentspc, (int)'\t', ntabs);
+            memset (indentspc + ntabs, (int)' ', nspcs);
+        }
+        else {
+            memset (indentspc, (int)' ', indentspclen);
+        }
+        indentspc[indentspclen] = '\0';
     }
     else {
         indentspc = (char *) strdup ("");
@@ -999,7 +1031,8 @@ int output_box (const sentry_t *thebox)
     for (j=skip_start; j<nol-skip_end; ++j) {
 
         if (j < thebox[BTOP].height) {   /* box top */
-            snprintf (obuf, LINE_MAX+1, "%s%s%s%s", indentspc,
+            restored_indent = tabbify_indent (0, indentspc, indentspclen);
+            concat_strings (obuf, LINE_MAX+1, 4, restored_indent,
                     skip_left?"":thebox[BLEF].chars[j], thebox[BTOP].chars[j],
                     thebox[BRIG].chars[j]);
         }
@@ -1007,7 +1040,8 @@ int output_box (const sentry_t *thebox)
         else if (vfill1) {               /* top vfill */
             r = thebox[BTOP].width;
             trailspc[r] = '\0';
-            snprintf (obuf, LINE_MAX+1, "%s%s%s%s", indentspc,
+            restored_indent = tabbify_indent (0, indentspc, indentspclen);
+            concat_strings (obuf, LINE_MAX+1, 4, restored_indent,
                     skip_left?"":thebox[BLEF].chars[j], trailspc,
                     thebox[BRIG].chars[j]);
             trailspc[r] = ' ';
@@ -1026,7 +1060,8 @@ int output_box (const sentry_t *thebox)
                     return rc;
                 r = input.maxline - input.lines[ti].len;
                 trailspc[r] = '\0';
-                snprintf (obuf, LINE_MAX+1, "%s%s%s%s%s%s%s", indentspc,
+                restored_indent = tabbify_indent (ti, indentspc, indentspclen);
+                concat_strings (obuf, LINE_MAX+1, 7, restored_indent,
                         skip_left?"":thebox[BLEF].chars[j], hfill1,
                         ti >= 0? input.lines[ti].text : "", hfill2,
                         trailspc, thebox[BRIG].chars[j]);
@@ -1034,7 +1069,8 @@ int output_box (const sentry_t *thebox)
             else {                       /* bottom vfill */
                 r = thebox[BTOP].width;
                 trailspc[r] = '\0';
-                snprintf (obuf, LINE_MAX+1, "%s%s%s%s", indentspc,
+                restored_indent = tabbify_indent (input.anz_lines - 1, indentspc, indentspclen);
+                concat_strings (obuf, LINE_MAX+1, 4, restored_indent,
                         skip_left?"":thebox[BLEF].chars[j], trailspc,
                         thebox[BRIG].chars[j]);
             }
@@ -1042,7 +1078,8 @@ int output_box (const sentry_t *thebox)
         }
 
         else {                           /* box bottom */
-            snprintf (obuf, LINE_MAX+1, "%s%s%s%s", indentspc,
+            restored_indent = tabbify_indent (input.anz_lines - 1, indentspc, indentspclen);
+            concat_strings (obuf, LINE_MAX+1, 4, restored_indent,
                     skip_left?"":thebox[BLEF].chars[j],
                     thebox[BBOT].chars[j-(nol-thebox[BBOT].height)],
                     thebox[BRIG].chars[j]);
@@ -1057,6 +1094,9 @@ int output_box (const sentry_t *thebox)
         else {
             btrim (obuf, &obuf_len);
         }
+        if (opt.tabexp == 'k') {
+            BFREE (restored_indent);
+        }
 
         fprintf (opt.outfile, "%s\n", obuf);
     }
