@@ -2,11 +2,10 @@
  *  File:             remove.c
  *  Project Main:     boxes.c
  *  Date created:     June 23, 1999 (Wednesday, 20:59h)
- *  Author:           Copyright (C) 1999 Thomas Jensen
- *                    tsjensen@stud.informatik.uni-erlangen.de
- *  Version:          $Id: remove.c,v 1.6 1999/11/08 10:51:41 tsjensen Exp tsjensen $
+ *  Author:           Copyright (C) 1999 Thomas Jensen <boxes@thomasjensen.com>
+ *  Version:          $Id: remove.c,v 1.8 2006/07/22 19:19:26 tsjensen Exp $
  *  Language:         ANSI C
- *  World Wide Web:   http://home.pages.de/~jensen/boxes/
+ *  World Wide Web:   http://boxes.thomasjensen.com/
  *  Purpose:          Box removal, i.e. the deletion of boxes
  *
  *  Remarks: o This program is free software; you can redistribute it and/or
@@ -25,7 +24,15 @@
  *  Revision History:
  *
  *    $Log: remove.c,v $
- *    Revision 1.6  1999/11/08 10:51:41  tsjensen
+ *    Revision 1.8  2006/07/22 19:19:26  tsjensen
+ *    Applied patch by Christoph Dreyer to support unexpansion of leading tabs
+ *    Added ability to retain existing tabs
+ *
+ *    Revision 1.7  2006/07/12 05:28:58  tsjensen
+ *    Updated email and web addresses in comment header
+ *    Added trim_only flag to output_input() function, used for box mending
+ *
+ *    Revision 1.6  1999-11-08 02:51:41-08  tsjensen
  *    Bugfix: For non-empty left box sides, spaces belonging to "empty" shape
  *    lines were not properly removed in some cases
  *
@@ -62,7 +69,7 @@
 #include "remove.h"
 
 static const char rcsid_remove_c[] =
-    "$Id: remove.c,v 1.6 1999/11/08 10:51:41 tsjensen Exp tsjensen $";
+    "$Id: remove.c,v 1.8 2006/07/22 19:19:26 tsjensen Exp $";
 
 
 
@@ -1040,7 +1047,7 @@ int remove_box()
 
 
 
-void output_input()
+void output_input (const int trim_only)
 /*
  *  Output contents of input line list "as is" to standard output, except
  *  for removal of trailing spaces (trimming).
@@ -1049,12 +1056,46 @@ void output_input()
  */
 {
     size_t j;
+    size_t indent;
+    char  *indentspc;
+    int    ntabs, nspcs;
 
-    for (j=0; j<input.anz_lines; ++j) {
-        if (input.lines[j].text) {
-            btrim (input.lines[j].text, &(input.lines[j].len));
-            printf ("%s\n", input.lines[j].text);
+    #ifdef DEBUG
+        fprintf (stderr, "output_input() - enter (trim_only=%d)\n", trim_only);
+    #endif
+    for (j=0; j<input.anz_lines; ++j)
+    {
+        if (input.lines[j].text == NULL)
+            continue;
+        btrim (input.lines[j].text, &(input.lines[j].len));
+        if (trim_only)
+            continue;
+        
+        indentspc = NULL;
+        if (opt.tabexp == 'u') {
+            indent = strspn (input.lines[j].text, " ");
+            ntabs = indent / opt.tabstop;
+            nspcs = indent % opt.tabstop;
+            indentspc = (char *) malloc (ntabs + nspcs + 1);
+            if (indentspc == NULL) {
+                perror (PROJECT);
+                return;
+            }
+            memset (indentspc, (int)'\t', ntabs);
+            memset (indentspc+ntabs, (int)' ', nspcs);
+            indentspc[ntabs+nspcs] = '\0';
         }
+        else if (opt.tabexp == 'k') {
+            indentspc = tabbify_indent (j, NULL, input.indent);
+            indent = input.indent;
+        }
+        else {
+            indentspc = (char *) strdup ("");
+            indent = 0;
+        }
+        
+        printf ("%s%s\n", indentspc, input.lines[j].text + indent);
+        BFREE (indentspc);
     }
 }
 
